@@ -324,7 +324,10 @@ theme_adium_parse_body (EmpathyThemeAdium *theme,
 			if (s > last) {
 				/* Append the text between last link (or the
 				 * start of the message) and this link */
-				g_string_append_len (string, text + last, s - last);
+				gchar *str;
+				str = g_markup_escape_text (text + last, s - last);
+				g_string_append (string, str);
+				g_free (str);
 			}
 
 			/* Append the link inside <a href=""></a> tag */
@@ -342,7 +345,10 @@ theme_adium_parse_body (EmpathyThemeAdium *theme,
 
 		if (e < strlen (text)) {
 			/* Append the text after the last link */
-			g_string_append_len (string, text + e, strlen (text) - e);
+			gchar *str;
+			str = g_markup_escape_text (text + e, strlen (text) - e);
+			g_string_append (string, str);
+			g_free (str);
 		}
 
 		g_free (ret);
@@ -372,6 +378,8 @@ theme_adium_parse_body (EmpathyThemeAdium *theme,
 
 		g_free (ret);
 		text = ret = g_string_free (string, FALSE);
+	} else {
+		text = ret = g_markup_escape_text (text, -1);
 	}
 
 	g_match_info_free (match_info);
@@ -544,7 +552,8 @@ theme_adium_append_message (EmpathyChatView *view,
 	const gchar           *func;
 	const gchar           *service_name;
 	GString               *message_classes = NULL;
-	gboolean              is_backlog;
+	gboolean               is_backlog;
+	gboolean               consecutive;
 
 	if (!priv->page_loaded) {
 		priv->message_queue = g_list_prepend (priv->message_queue,
@@ -600,77 +609,58 @@ theme_adium_append_message (EmpathyChatView *view,
 		}
 	}
 
+	/* We want to join this message with the last one if
+	 * - senders are the same contact,
+	 * - last message was recieved recently,
+	 * - last message and this message both are/aren't backlog, and
+	 * - DisableCombineConsecutive is not set in theme's settings */
 	is_backlog = empathy_message_is_backlog (msg);
+	consecutive = empathy_contact_equal (priv->last_contact, sender) &&
+		(timestamp - priv->last_timestamp < MESSAGE_JOIN_PERIOD) &&
+		(is_backlog == priv->last_is_backlog) &&
+		!tp_asv_get_boolean (priv->data->info,
+				     "DisableCombineConsecutive", NULL);
 
-	/* Get the right html/func to add the message */
-	func = "appendMessage";
-
+	/* Define message classes */
 	message_classes = g_string_new ("message");
-
-	/* eventually append the "history" class */
 	if (is_backlog) {
 		g_string_append (message_classes, " history");
 	}
-
-	/* check the sender of the message and append the appropriate class */
+	if (consecutive) {
+		g_string_append (message_classes, " consecutive");
+	}
 	if (empathy_contact_is_user (sender)) {
 		g_string_append (message_classes, " outgoing");
-	}
-	else {
+	} else {
 		g_string_append (message_classes, " incoming");
 	}
 
-	/*
-	 * To mimick Adium's behavior, we only want to join messages
-	 * sent by the same contact within a 5 minute time frame.
-	 */
-	if (empathy_contact_equal (priv->last_contact, sender) &&
-	    (timestamp - priv->last_timestamp < MESSAGE_JOIN_PERIOD) &&
-	    (is_backlog == priv->last_is_backlog)) {
-		/* the messages can be appended */
+	/* Define javascript function to use */
+	if (consecutive) {
 		func = "appendNextMessage";
-		g_string_append (message_classes, " consecutive");
+	} else {
+		func = "appendMessage";
+	}
 
-		/* check who is the sender of the message to use the correct html file */
-		if (empathy_contact_is_user (sender)) {
-			/* check if this is a backlog message and use NextContext.html */
+	/* Outgoing */
+	if (empathy_contact_is_user (sender)) {
+		if (consecutive) {
 			if (is_backlog) {
 				html = priv->data->out_nextcontext_html;
 				len = priv->data->out_nextcontext_len;
 			}
 
-			/*
-			 * html is null if this is not a backlog message or
-			 * if we have to fallback (NextContext.html missing).
-			 * use NextContent.html
-			 */
+			/* Not backlog, or fallback if NextContext.html
+			 * is missing */
 			if (html == NULL) {
 				html = priv->data->out_nextcontent_html;
 				len = priv->data->out_nextcontent_len;
 			}
 		}
-		else {
-			if (is_backlog) {
-				html = priv->data->in_nextcontext_html;
-				len = priv->data->in_nextcontext_len;
-			}
 
-			if (html == NULL) {
-				html = priv->data->in_nextcontent_html;
-				len = priv->data->in_nextcontent_len;
-			}
-		}
-	}
-
-	/*
-	 * we have html == NULL here if:
-	 * 1. the message didn't have to be appended because
-	 *    the sender was different or the timestamp was too far
-	 * 2. NextContent.html file does not exist, so we must
-	 *    not forget to fallback to the correct Content.html
-	 */
-	if (html == NULL) {
-		if (empathy_contact_is_user (sender)) {
+		/* Not consecutive, or fallback if NextContext.html and/or
+		 * NextContent.html are missing */
+		if (html == NULL) {
 			if (is_backlog) {
 				html = priv->data->out_context_html;
 				len = priv->data->out_context_len;
@@ -681,7 +671,27 @@ theme_adium_append_message (EmpathyChatView *view,
 				len = priv->data->out_content_len;
 			}
 		}
-		else {
+	}
+
+	/* Incoming, or fallback if outgoing files are missing */
+	if (html == NULL) {
+		if (consecutive) {
+			if (is_backlog) {
+				html = priv->data->in_nextcontext_html;
+				len = priv->data->in_nextcontext_len;
+			}
+
+			/* Note backlog, or fallback if NextContext.html
+			 * is missing */
+			if (html == NULL) {
+				html = priv->data->in_nextcontent_html;
+				len = priv->data->in_nextcontent_len;
+			}
+		}
+
+		/* Not consecutive, or fallback if NextContext.html and/or
+		 * NextContent.html are missing */
+		if (html == NULL) {
 			if (is_backlog) {
 				html = priv->data->in_context_html;
 				len = priv->data->in_context_len;
@@ -694,9 +704,14 @@ theme_adium_append_message (EmpathyChatView *view,
 		}
 	}
 
-	theme_adium_append_html (theme, func, html, len, body, avatar_filename,
-				 name, contact_id, service_name, message_classes->str,
-				 timestamp);
+	if (html != NULL) {
+		theme_adium_append_html (theme, func, html, len, body,
+					 avatar_filename, name, contact_id,
+					 service_name, message_classes->str,
+					 timestamp);
+	} else {
+		DEBUG ("Couldn't find HTML file for this message");
+	}
 
 	/* Keep the sender of the last displayed message */
 	if (priv->last_contact) {
@@ -718,11 +733,15 @@ theme_adium_append_event (EmpathyChatView *view,
 	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
 
 	if (priv->data->status_html) {
+		gchar *str_escaped;
+
+		str_escaped = g_markup_escape_text (str, -1);
 		theme_adium_append_html (theme, "appendMessage",
 					 priv->data->status_html,
 					 priv->data->status_len,
-					 str, NULL, NULL, NULL, NULL, "event",
-					 empathy_time_get_current ());
+					 str_escaped, NULL, NULL, NULL, NULL,
+					 "event", empathy_time_get_current ());
+		g_free (str_escaped);
 	}
 
 	/* There is no last contact */
