@@ -38,8 +38,6 @@
 #include <gtk/gtk.h>
 #include <gio/gio.h>
 
-#include <libmissioncontrol/mc-profile.h>
-
 #include "empathy-ui-utils.h"
 #include "empathy-images.h"
 #include "empathy-conf.h"
@@ -53,8 +51,8 @@
 
 #define SCHEMES "(https?|s?ftps?|nntp|news|javascript|about|ghelp|apt|telnet|"\
 		"file|webcal|mailto)"
-#define BODY "([^\\ \\n]+)"
-#define END_BODY "([^\\ \\n]*[^,;\?><()\\ \"\\.\\n])"
+#define BODY "([^\\ \\n\"]+)"
+#define END_BODY "([^\\ \\n\"]*[^,;\?><()\\ \"\\.\\n])"
 #define URI_REGEX "("SCHEMES"://"END_BODY")" \
 		  "|((mailto:)?"BODY"@"BODY"\\."END_BODY")"\
 		  "|((www|ftp)\\."END_BODY")"
@@ -101,9 +99,19 @@ builder_get_file_valist (const gchar *filename,
 
 	gui = gtk_builder_new ();
 	if (!gtk_builder_add_from_file (gui, filename, &error)) {
-		g_critical ("GtkBuilder Error: %s", error->message);
+		g_critical ("GtkBuilder Error (%s): %s",
+				filename, error->message);
 		g_clear_error (&error);
 		g_object_unref (gui);
+
+		/* we need to iterate and set all of the pointers to NULL */
+		for (name = first_object; name;
+		     name = va_arg (args, const gchar *)) {
+			object_ptr = va_arg (args, GObject**);
+
+			*object_ptr = NULL;
+		}
+
 		return NULL;
 	}
 
@@ -182,16 +190,6 @@ empathy_builder_unref_and_keep_widget (GtkBuilder *gui,
 	return widget;
 }
 
-const gchar *
-empathy_icon_name_from_account (EmpathyAccount *account)
-{
-	McProfile *profile;
-
-	profile = empathy_account_get_profile (account);
-
-	return mc_profile_get_icon_name (profile);
-}
-
 const gchar *
 empathy_icon_name_for_presence (TpConnectionPresenceType presence)
 {
@@ -1297,17 +1295,15 @@ empathy_window_present (GtkWindow *window,
 			gboolean   steal_focus)
 {
 	guint32 timestamp;
+	GdkWindow *gdk_window;
 
 	g_return_if_fail (GTK_IS_WINDOW (window));
 
-	/* There are three cases: hidden, visible, visible on another
-	 * workspace.
-	 */
-
-	if (!empathy_window_get_is_visible (window)) {
-		/* Hide it so present brings it to the current workspace. */
-		gtk_widget_hide (GTK_WIDGET (window));
-	}
+	/* Move the window to the current workspace before trying to show it.
+	 * This is the behaviour people expect when clicking on the statusbar icon. */
+	gdk_window = gtk_widget_get_window (GTK_WIDGET (window));
+	if (gdk_window)
+		gdk_x11_window_move_to_current_desktop (gdk_window);
 
 	timestamp = gtk_get_current_event_time ();
 	gtk_window_present_with_time (window, timestamp);
@@ -1333,25 +1329,46 @@ empathy_get_toplevel_window (GtkWidget *widget)
 	return NULL;
 }
 
-/* The URL opening code can't handle schemeless strings, so we try to be
- * smart and add http if there is no scheme or doesn't look like a mail
- * address. This should work in most cases, and let us click on strings
- * like "www.gnome.org".
+/** empathy_make_absolute_url_len:
+ * @url: an url
+ * @len: a length
+ *
+ * Same as #empathy_make_absolute_url but for a limited string length
  */
-static gchar *
-fixup_url (const gchar *url)
+gchar *
+empathy_make_absolute_url_len (const gchar *url,
+			       guint len)
 {
+	g_return_val_if_fail (url != NULL, NULL);
+
 	if (g_str_has_prefix (url, "ghelp:") ||
 	    g_str_has_prefix (url, "mailto:") ||
 	    strstr (url, ":/")) {
-		return NULL;
+		return g_strndup (url, len);
 	}
 
 	if (strstr (url, "@")) {
-		return g_strdup_printf ("mailto:%s", url);
+		return g_strdup_printf ("mailto:%.*s", len, url);
 	}
 
-	return g_strdup_printf ("http://%s", url);
+	return g_strdup_printf ("http://%.*s", len, url);
+}
+
+/** empathy_make_absolute_url:
+ * @url: an url
+ *
+ * The URL opening code can't handle schemeless strings, so we try to be
+ * smart and add http if there is no scheme or doesn't look like a mail
+ * address. This should work in most cases, and let us click on strings
+ * like "www.gnome.org".
+ *
+ * Returns: a newly allocated url with proper mailto: or http:// prefix, use
+ * g_free when your are done with it
+ */
+gchar *
+empathy_make_absolute_url (const gchar *url)
+{
+	return empathy_make_absolute_url_len (url, strlen (url));
 }
 
 void
@@ -1361,12 +1378,12 @@ empathy_url_show (GtkWidget *parent,
 	gchar  *real_url;
 	GError *error = NULL;
 
-	real_url = fixup_url (url);
-	if (real_url) {
-		url = real_url;
-	}
+	g_return_if_fail (parent == NULL || GTK_IS_WIDGET (parent));
+	g_return_if_fail (url != NULL);
 
-	gtk_show_uri (gtk_widget_get_screen (parent), url,
+	real_url = empathy_make_absolute_url (url);
+
+	gtk_show_uri (parent ? gtk_widget_get_screen (parent) : NULL, real_url,
 		      gtk_get_current_event_time (), &error);
 
 	if (error) {
@@ -1483,6 +1500,8 @@ empathy_send_file_with_file_chooser (EmpathyContact *contact)
 	gtk_dialog_set_default_response (GTK_DIALOG (widget),
 					 GTK_RESPONSE_OK);
 
+	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (widget), FALSE);
+
 	g_signal_connect (widget, "response",
 			  G_CALLBACK (file_manager_send_file_response_cb),
 			  contact);
@@ -1539,4 +1558,4 @@ empathy_receive_file_with_file_chooser (EmpathyFTHandler *handler)
 		G_CALLBACK (file_manager_receive_file_response_cb), handler);
 
 	gtk_widget_show (widget);
-}
\ No newline at end of file
+}
