@@ -130,7 +130,7 @@ notification_close_helper (EmpathyStatusIconPriv *priv)
 	}
 }
 
-#ifdef HAVE_NBTK
+#ifdef HAVE_MOBLIN
 static void
 notification_action_cb (NotifyNotification *notification,
 			gchar              *action,
@@ -157,21 +157,25 @@ status_icon_update_notification (EmpathyStatusIcon *icon)
 
 	if (priv->event) {
 		gchar *message_esc = NULL;
+		gchar *header_esc = NULL;
 
 		if (priv->event->message != NULL)
 			message_esc = g_markup_escape_text (priv->event->message, -1);
 
+		if (priv->event->header != NULL)
+			header_esc = g_markup_escape_text (priv->event->header, -1);
+
 		if (priv->notification) {
 			notify_notification_update (priv->notification,
-						    priv->event->header, message_esc,
+						    header_esc, message_esc,
 						    NULL);
 		} else {
 			priv->notification = notify_notification_new_with_status_icon
-				(priv->event->header, message_esc, NULL, priv->icon);
+				(header_esc, message_esc, NULL, priv->icon);
 			notify_notification_set_timeout (priv->notification,
 							 NOTIFY_EXPIRES_DEFAULT);
 
-#ifdef HAVE_NBTK
+#ifdef HAVE_MOBLIN
 			if (priv->notify_supports_actions) {
 				notify_notification_add_action (priv->notification,
 					"respond",
@@ -198,6 +202,7 @@ status_icon_update_notification (EmpathyStatusIcon *icon)
 		notify_notification_show (priv->notification, NULL);
 
 		g_free (message_esc);
+		g_free (header_esc);
 	} else {
 		notification_close_helper (priv);
 	}
@@ -238,6 +243,13 @@ status_icon_update_icon (EmpathyStatusIcon *icon)
 		TpConnectionPresenceType state;
 
 		state = empathy_idle_get_state (priv->idle);
+
+		/* An unset presence type here doesn't make sense. Force it
+		 * to be offline. */
+		if (state == TP_CONNECTION_PRESENCE_TYPE_UNSET) {
+			state = TP_CONNECTION_PRESENCE_TYPE_OFFLINE;
+		}
+
 		icon_name = empathy_icon_name_for_presence (state);
 	}
 
